% Example5_10.m
% Computes the N-point Discrete Cosine,
% and Hadamard Transforms of an N-point input
% sequence. It also computes the percentage of
% total energy contained in each transform coefficien.
% Author: K.S. Thyagarajan
%
clear
N = 32; % number of samples in input sequence
n = 0:N-1;
f1 = 10; % frequency f1 in Hz
f2 = 23; % frequency f2 in Hz
f3 = 33; % frequency f3 in Hz
fs = 100; % sampling frequency in Hz
w1 = 2*pi*f1/fs;
w2 = 2*pi*f2/fs;
w3 = 2*pi*f3/fs;
x = 1.5*sin(w1*n)-0.5*cos(w2*n)+sin(w3*n); % N-point input sequence
%
X_dct = dct(x); % DCT II of input sequence
X_h = fwht(x); % Hadamard transform of input sequence
%
E_dct = X_dct.*X_dct; % Energy in DCT coefficients
E_h = X_h.*X_h; % Energy in Hadamard coefficients
%
y1 = idct(X_dct); % inverse DCT
y2 = ifwht(X_h); % inverse Hadamard transform
%
% Plot the input sequence and corresponding transforms
figure,subplot(3,1,1),stem(n,x,'k')
title('Input sequence'), ylabel('Amplitude')
subplot(3,1,2),stem(n,X_dct,'k')
title('DCT of input sequence'), ylabel('Amplitude')
subplot(3,1,3),stem(n,y1,'k')
title('IDCT of DCT of input sequence'), ylabel('Amplitude')
xlabel('Sample index')
%
figure,subplot(3,1,1),stem(n,x,'k')
title('Input sequence'), ylabel('Amplitude')
subplot(3,1,2),stem(n,X_h,'k')
title('FWHT of input sequence'), ylabel('Amplitude')
subplot(3,1,3),stem(n,y2,'k')
title('IFWHT of FWHT of input sequence'), ylabel('Amplitude')
xlabel('Sample index')
% Plot the percentage of total energy
% contained in the transform coefficients
figure,subplot(2,1,1),bar((E_dct/sum(E_dct))*100,'k')
title('Energy in DCT coefficients')
ylabel('% total energy')
subplot(2,1,2),bar((E_h/sum(E_h))*100,'k')
title('Energy in Hadamard transform coefficients')
ylabel('% total energy'), xlabel('Frequency index k')