% Example5_2.m
% Example illustrating the relationship
% between frequency sampling and
% discrete-time sequence
% Author: K.S. Thyagarajan
clear
N = 32; % sequence length
%M = 32;
M = 8; % # samples in the DTFT
NbyM = floor(N/M);
x = sin(0.2*pi*(0:N-1))+2*cos(0.3*pi*(0:N-1));
A = zeros(size(x)); A(1) = 1;
[H,W] = freqz(x,A,N,'whole');% DTFT of the sequence x
h = ifft(H); % N-point IDFT of M = N
X = H(1:NbyM:N);% sample the DTFT at M < N points
y = ifft(X);% IDFT of sampled DTFT of x
% stem plot the sequences
figure,subplot(2,1,1),stem(0:N-1,x,'k')
title([int2str(N) '-point sequence x[n]'])
ylabel('Amplitude')
subplot(2,1,2),stem(0:N-1,h,'k')
title(['sequence from ' int2str(N) '-sample DFT'])
ylabel('Amplitude'), xlabel('Sample index, n')
%
figure,subplot(2,1,1),stem(0:N-1,x,'k')
title([int2str(N) '-point sequence x[n]'])
ylabel('Amplitude')
subplot(2,1,2),stem(0:M-1,real(y),'k')
title(['sequence from ' int2str(M) '-sample DFT'])
ylabel('Amplitude'), xlabel('Sample index, n')
%
% Reconstruction in the discrete-time domain by
% summing the replications of the original N-point
% sequence
z = x(1:M) + x(M+1:2*M) + x(2*M+1:3*M) + x(3*M+1:4*M);
figure,subplot(2,1,1),stem(0:M-1,real(y),'k')
title('y[n]=x[n]+x[n+2M]+x[n+3M]+x[n+4M]')
ylabel('Amplitude'),legend('from IDFT')
subplot(2,1,2),stem(0:M-1,z,'k')
ylabel('Amplitude'), xlabel('Sample index(n)')
legend('from replicas')
% Plot the spectra of the input {x[n]} and reconstructed sequence {z[n]}
Xw = freqz(x,A,256,'whole'); % 256-point DTFT of x[n]
zA = zeros(size(z)); zA(1) = 1;
Zw = freqz(z,zA,256,'whole');
figure,subplot(2,1,1),plot((2*pi/256)*(0:255),abs(Xw)/max(abs(Xw)),'k')
title('DTFT of input sequence')
ylabel('Normalized magnitude')
subplot(2,1,2),plot((2*pi/256)*(0:255),abs(Zw)/max(abs(Zw)),'k')
title('DTFT of reconstructed sequence')
ylabel('Normalized magnitude'), xlabel('Normalized frequency, rad')