% Overlap_save.m
% Linear convolution of a long sequence with a
% much smaller length sequence by overlap-save method
% Author: K.S. Thyagarajan
clear
% design a lowpass FIR filter of length N with a
% normalized cutoff frequency wn
N = 15;% FIR filter length
wn = 0.25;% filter cutoff frequency normalized to 
% half the sampling frequency
W = window(@hamming,N);% Hamming window
b1 = fir1(N-1,wn,W); % N-1st-order FIR LPF using Hamming window
f1 = 200;% a frequency present in the input sequence
f2 = 1100;% a second frequency present in the input sequence
Fs = 5000;% sampling frequency
w1 = 2*pi*f1/Fs;
w2 = 2*pi*f2/Fs;
% plot the magnitude of the frequency response of the FIR filter
a = zeros(size(b1)); a(1) = 1;
[H,W] = freqz(b1,a,256);
figure,plot((Fs/256)*(0:255),20*log10(abs(H)),'k')
title('Frquency response of FIR filter')
ylabel('Magnitude(dB)')
xlabel('Frequency(Hz)')
%
M = 50;% length of each overlapping input block
L = M + N -1;% length of linear convolution
SL = 2048;% length of input sequence
n = 0:ceil(SL/M)*M-1;% sample index
x = 1.5*cos(w1*n) + 2*cos(w2*n);% input sequence
X = fft(x,SL);% DFT of the input sequence
% zero-pad filter with M zeros
b = zeros(1,L);
b(1:N)=b1;
B = fft(b);% one time DFT of the filter
x1 = zeros(1,L);
y = zeros(size(x));% output sequence
for k = 0:floor(SL/M)
    if k == 0
        x1(N:L) = x(1:M);% 1st N-1 samples are zero
    else
        k1 = k*M - N + 2; k2 = k1 + L - 1;
        x1 = x(k1:k2);% 1st N-1 samples in current block overlap with
                      % previous N-1 samples
    end
    y1 = ifft(B .* fft(x1,L));% linear conv of FIR filter &
                              % current block
    if k == 0
        y(1:M) = y1(N:L);
    else
        l1 = k*M + 1; l2 = l1 + M - 1;
        y(l1:l2)=y1(N:L);% discard 1st N-1 samples in the output
    end
end
Y = fft(y,SL);
figure,subplot(2,1,1),plot(0:255,x(1:256),'k')
title('Input sequence')
ylabel('Amplitude')
subplot(2,1,2),plot(0:255,y(1:256),'k')
title('Filtered sequence')
ylabel('Amplitude'), xlabel('Sample index(n)')
figure,subplot(2,1,1),plot(Fs*(0:SL-1)/SL,abs(X)/max(abs(X)),'k')
title('Spetrum of input sequence')
ylabel('Magnitude')
subplot(2,1,2),plot(Fs*(0:SL-1)/SL,abs(Y)/max(abs(Y)),'k')
title('Spetrum of filtered sequence')
ylabel('Magnitude'), xlabel('Freqyency(Hz)')
